from django.db import models
from django.db.models import CharField, TextField
from django.urls import reverse

from django_quill.fields import QuillField

__all__ = (
    "QuillPost",
    "NonQuillPost",
)


class QuillPost(models.Model):
    content = QuillField()

    class Meta:
        ordering = ["-pk"]

    def get_absolute_url(self):
        return reverse("posts:quill-post-detail", args=[self.pk])


class NonQuillPost(models.Model):
    content_char = CharField(max_length=300, blank=True)
    content_text = TextField(blank=True)



# Generated by Django 3.2.9 on 2021-12-30 04:49

from django.db import migrations, models

DATAS = [
    {
        "content_char": "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut nunc dolor, malesuada in pulvinar et, efficitur non massa. Sed at mi elit. Quisque a dapibus nibh. Nunc ac tristique mi. Nam consectetur lectus ac ipsum euismod ultrices. Interdum et malesuada fames ac ante ipsum primis in faucibus. Mauris condimentum, libero vitae rutrum tristique, nibh urna maximus felis, ac gravida enim velit id erat.",
    },
    {
        "content_text": "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut nunc dolor, malesuada in pulvinar et, efficitur non massa. Sed at mi elit. Quisque a dapibus nibh. Nunc ac tristique mi. Nam consectetur lectus ac ipsum euismod ultrices. Interdum et malesuada fames ac ante ipsum primis in faucibus. Mauris condimentum, libero vitae rutrum tristique, nibh urna maximus felis, ac gravida enim velit id erat.",
    },
]


def forwards_func(apps, schema_editor):
    NonQuillPost = apps.get_model("posts", "NonQuillPost")
    db_alias = schema_editor.connection.alias
    for data in DATAS:
        NonQuillPost.objects.using(db_alias).create(**data)


def reverse_func(apps, schema_editor):
    NonQuillPost = apps.get_model("posts", "NonQuillPost")
    db_alias = schema_editor.connection.alias
    NonQuillPost.objects.using(db_alias).all().delete()


class Migration(migrations.Migration):
    dependencies = [
        ("posts", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="NonQuillPost",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("content_char", models.CharField(blank=True, max_length=300)),
                ("content_text", models.TextField(blank=True)),
            ],
        ),
        migrations.RunPython(forwards_func, reverse_func),
    ]



from rest_framework import serializers

from django_quill.fields import FieldQuill

__all__ = (
    "QuillFieldMixin",
    "QuillHtmlField",
    "QuillPlainField",
)


class QuillFieldMixin:
    pass


class QuillHtmlField(QuillFieldMixin, serializers.Field):
    def to_representation(self, value: FieldQuill):
        return value.quill.html


class QuillPlainField(QuillFieldMixin, serializers.Field):
    def to_representation(self, value: FieldQuill):
        return value.quill.plain



from django.db.models.signals import post_save
from django.dispatch import receiver

from posts.models import QuillPost


@receiver(post_save, sender=QuillPost)
def posts_limit_10(sender, **kwargs):
    recent_posts = sender.objects.order_by("-id")[:10]
    sender.objects.exclude(id__in=recent_posts.values("id")).delete()



DEFAULT_CONFIG = {
    "theme": "snow",
    "modules": {
        "syntax": True,
        "toolbar": [
            [
                {"font": []},
                {"header": []},
                {"align": []},
                "bold",
                "italic",
                "underline",
                "strike",
                "blockquote",
                {"color": []},
                {"background": []},
            ],
            ["code-block", "link", "image", "video"],
            ["clean"],
        ],
        # quill-image-compress
        "imageCompressor": {
            "quality": 0.8,
            "maxWidth": 2000,
            "maxHeight": 2000,
            "imageType": "image/jpeg",
            "debug": False,
            "suppressErrorLogging": True,
        },
        # quill-resize
        "resize": {
            "showSize": True,
            "locale": {},
        },
    },
}
MEDIA_JS = [
    # syntax-highlight
    "https://cdnjs.cloudflare.com/ajax/libs/highlight.js/10.1.1/highlight.min.js",
    # quill
    "https://cdn.quilljs.com/1.3.7/quill.min.js",
    # quill-image-compress
    "https://cdn.jsdelivr.net/npm/quill-image-compress@1.2.21/dist/quill.imageCompressor.min.js",
    # quill-resize
    "https://cdn.jsdelivr.net/npm/@botom/quill-resize-module@2.0.0/dist/quill-resize-module.min.js",
    # custom
    "django_quill/django_quill.js",
]
MEDIA_CSS = [
    # syntax-highlight
    "https://cdn.quilljs.com/1.3.7/quill.snow.css",
    "https://cdnjs.cloudflare.com/ajax/libs/highlight.js/10.1.1/styles/darcula.min.css",
    # quill-resize
    "https://cdn.jsdelivr.net/npm/quill-resize-module@1.2.4/dist/resize.min.css",
    # custom
    "django_quill/django_quill.css",
]



from rest_framework import serializers

from django_quill.drf.fields import QuillHtmlField, QuillPlainField
from posts.models import QuillPost

__all__ = ("QuillPostSerializer",)


class QuillPostSerializer(serializers.ModelSerializer):
    content = QuillHtmlField()
    content_plain = QuillPlainField(source="content")

    class Meta:
        model = QuillPost
        fields = "__all__"



from rest_framework import viewsets

from posts.models import QuillPost
from posts.serializers import QuillPostSerializer

__all__ = ("QuillPostViewSet",)


class QuillPostViewSet(viewsets.ModelViewSet):
    queryset = QuillPost.objects.all()
    serializer_class = QuillPostSerializer



"""config URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""

import importlib.util

from django.contrib import admin
from django.urls import path, include
from rest_framework.routers import SimpleRouter

from . import views

__all__ = ("urlpatterns",)

urlpatterns = [
    path("admin/", admin.site.urls),
    path("", views.IndexView.as_view(), name="index"),
    path("reset/", views.ResetView.as_view(), name="reset"),
    path("posts/", include("posts.urls")),
]
urlpatterns_apis = []

if (spec := importlib.util.find_spec("rest_framework")) is not None:
    from posts import apis

    router = SimpleRouter()
    router.register(r"", apis.QuillPostViewSet)
    urlpatterns_apis.append(path("posts/", include(router.urls)))

if urlpatterns_apis:
    urlpatterns.append(path("api/", include(urlpatterns_apis)))



import json
from json.decoder import JSONDecodeError

from django.apps import apps
from django.core.management import BaseCommand

from django_quill.fields import FieldQuill
from django_quill.quill import QuillParseError


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument("app_label", type=str)
        parser.add_argument("model_name", type=str)
        parser.add_argument("field_name", type=str)

    def handle(self, *args, **options):
        app_label = options["app_label"]
        model_name = options["model_name"]
        field_name = options["field_name"]

        model = apps.get_registered_model(app_label, model_name)
        instances = model.objects.only(field_name)
        for index, instance in enumerate(instances, start=1):
            print(
                f"[{index}/{len(instances)}] {model_name} (pk: {instance.pk}) converting"
            )
            field_data = getattr(instance, field_name)
            if isinstance(field_data, FieldQuill):
                try:
                    quill = field_data.quill
                    print(f" This is already a QuillField.")
                    continue
                except QuillParseError:
                    field_data = field_data.json_string
            try:
                json_data = json.loads(field_data)
                if "delta" in json_data:
                    print(f" Already in Quill format. has been skipped")
                    continue
            except JSONDecodeError:
                print(f" Converted ({field_data[:20]}...)")
            except TypeError:
                print(f" Change type ({type(field_data)} -> str)")
                field_data = str(field_data)
                print(f" Changed value: {field_data[:20]}...")
            finally:
                converted_data = {
                    "delta": "",
                    "html": field_data,
                }
                setattr(instance, field_name, json.dumps(converted_data))
                instance.save(update_fields=[field_name])
                print(f" Saved")



from django.core import management
from django.test import TestCase
from django.urls import reverse

from config.baker import baker
from django_quill.quill import Quill, QuillParseError
from posts.models import QuillPost, NonQuillPost


class QuillViewTest(TestCase):
    def test_generic_list_view(self):
        baker.make(QuillPost, _quantity=30)
        response = self.client.get(reverse("posts:quill-post-list"))
        self.assertEqual(response.status_code, 200)

    def test_generic_create_view(self):
        response = self.client.get(reverse("posts:quill-post-create"))
        self.assertEqual(response.status_code, 200)

    def test_generic_detail_view(self):
        quill_post = baker.make(QuillPost)
        response = self.client.get(
            reverse("posts:quill-post-detail", args=[quill_post.id])
        )
        self.assertEqual(response.status_code, 200)


class QuillAdminTest(TestCase):
    def setUp(self) -> None:
        self.client.login()

    def test_list(self):
        baker.make(QuillPost, _quantity=13)
        url = reverse("admin:posts_quillpost_changelist")
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_detail(self):
        post = baker.make(QuillPost)
        url = reverse("admin:posts_quillpost_change", args=[post.id])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)


class ConvertToQuillCommandTest(TestCase):
    def test_command(self):
        dummy_text = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut nunc dolor, malesuada in pulvinar et, efficitur non massa. Sed at mi elit. Quisque a dapibus nibh. Nunc ac tristique mi. Nam consectetur lectus ac ipsum euismod ultrices. Interdum et malesuada fames ac ante ipsum primis in faucibus. Mauris condimentum, libero vitae rutrum tristique, nibh urna maximus felis, ac gravida enim velit id erat."
        post_char = baker.make(NonQuillPost, content_char=dummy_text)
        post_text = baker.make(NonQuillPost, content_text=dummy_text)
        with self.assertRaises(QuillParseError):
            Quill(post_char.content_char)
        with self.assertRaises(QuillParseError):
            Quill(post_text.content_text)

        management.call_command(
            "convert_to_quill", "posts", "NonQuillPost", "content_char"
        )
        post_char.refresh_from_db()
        quill = Quill(post_char.content_char)
        self.assertEqual(quill.html, dummy_text)
        with self.assertRaises(QuillParseError):
            Quill(post_text.content_text)

        management.call_command(
            "convert_to_quill", "posts", "NonQuillPost", "content_text"
        )
        post_text.refresh_from_db()
        quill = Quill(post_text.content_text)
        self.assertEqual(quill.html, dummy_text)
